use std::{
    fs::create_dir,
    path::Path,
};

use crate::utils::*;

use serde::{Serialize, Deserialize};
use serde_json::Value;

use color_eyre::eyre::{eyre, Result};

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct Config {
    pub pk_key: String,
    pub sp_key: String,
    #[cfg(feature = "avatar")]
    pub avatar_module: AvatarModule,
    #[cfg(feature = "discord")]
    pub disc_module: DiscModule,
    #[cfg(feature = "fedi")]
    pub fedi_module: FediModule,
}

#[cfg(feature = "avatar")]
#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct AvatarModule {
    pub enabled: bool,
    pub avatar_folder: String,
    pub avatar_output_path: String,
    pub blacklist: Vec<String>,
}

#[cfg(feature = "discord")]
#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct DiscModule {
    pub enabled: bool,
    pub token: String,
    pub python_path: String,
    pub script_path: String,
}

#[cfg(feature = "fedi")]
#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct FediModule {
    pub enabled: bool,
    pub instance: String,
    pub token: String,
}

impl Config {
    pub fn get_config(cfg_dir: &str) -> Result<Config> {

        let cfg_file_path = format!("{}/config.json", cfg_dir);

        if !Path::new(&cfg_dir).exists() {
            let _ = create_dir(cfg_dir);
        }

        if Path::new(&cfg_file_path).exists() {
            let json_config: Value = load_json(&cfg_file_path)?;
            let conf: Config = serde_json::from_value(json_config)?;
            Ok(conf)
        } else {
            // TODO Trigger file creation screen if there is no config file.
            Err(eyre!("Config file not found"))
        }
    }
}
