mod app;
mod configdata;
mod systemdata;
mod api;
mod utils;
mod clap_ps;

use dirs;

use app::*;
use utils::update_avatars;

use clap_ps::*;
use clap::Parser;

use color_eyre::eyre::{eyre, Result};

fn main() -> Result<()> {

    let cli = Args::parse();

    let config_path = match dirs::config_dir() {
        Some(d) => format!("{}/pluralsync", d.display()),
        None => {
            return Err(eyre!("Could not figure out the config directory!"));
        }
    };

    let mut app: App = App::new(config_path.clone());
    app.load_config()?;
    app.load_system(false)?;

    match cli.cmd {
        Commands::Update => {
            app.load_system(true)?;
        },

        Commands::TUI => {
            ()
        },

        Commands::Set { members, #[cfg(feature = "discord")] discord, #[cfg(feature = "fedi")] fedi } => {
            app.set_member(members, false)?;

            #[cfg(feature = "avatar")]
            update_avatars(&mut app, #[cfg(feature = "discord")] discord, #[cfg(feature = "fedi")] fedi)?;
        },

        Commands::Add { members, #[cfg(feature = "discord")] discord, #[cfg(feature = "fedi")] fedi } => {
            app.set_member(members, true)?;

            #[cfg(feature = "avatar")]
            update_avatars(&mut app, #[cfg(feature = "discord")] discord, #[cfg(feature = "fedi")] fedi)?;
        },

        Commands::Get { force_from, #[cfg(feature = "discord")] discord, #[cfg(feature = "fedi")] fedi } => {
            let ff = match force_from {
                Some(x) => x,
                None => ForceFrom::None,
            };

            app.get_front(ff)?;

            #[cfg(feature = "avatar")]
            update_avatars(&mut app, #[cfg(feature = "discord")] discord, #[cfg(feature = "fedi")] fedi)?;
        },

        Commands::Members => {
            app.memberlist()?;
        },
    }
    Ok(())
}

